#include <array>
#include <cassert>
#include <iostream>
#include <chrono>
#include "./arrayMD.h"

#define N 1000

#if defined(_OPENMP) || defined(OPENMP_TARGET)
#include <omp.h>
#endif

using namespace std::chrono;

int main(int argc, char** argv)
{
  time_point<system_clock> start, end;
  duration<double> elapsed ;

  // Two implementations of MD arrays
  // 1 - uses variadic templates to generalize the dimension
  // 2 - a class for every dimension
#if VD
  ArrayMD<double,2> arr(N,N);
#else
  Array2D<double> arr(N,N);
#endif

#if defined(_OPENMP) && defined(OPENMP_TARGET)
#pragma omp target enter data map(to:arr)
#pragma omp target enter data map(to:arr.dptr[0:arr.size])
#endif

  start = system_clock::now();

  for(int nsteps = 0; nsteps < 1000; ++nsteps)
  {
#if defined(_OPENMP)
#if defined(OPENMP_TARGET)
#pragma omp target teams distribute parallel for collapse(2) //map(alloc:arr.dptr[:0])
#else
#pragma omp parallel for collapse(2) default(none) \
    shared(arr)
#endif
#endif
  for(int i = 0; i < N; ++i)
    for(int j = 0; j < N; ++j)
    {
      for(int k = 0; k < N; ++k)
      {
//        arr.dptr[i*N+j] += 0.5*i*j;
        arr(i,j) += 0.5*i*j;
      }
    }
  }

  end = system_clock::now();
  elapsed = end - start;

  std::cout << "Time taken by the kernel processing " << N*N << " elements = " << elapsed.count() << " seconds \n";

  return 0;
}
